# -*- coding: utf-8 -*-

from json import dumps as jsondumps

from django.contrib.auth.decorators import login_required
from django.core import serializers
from django.http import HttpResponse
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.views.decorators.csrf import csrf_exempt

from api.utils import APIObject, ScrobbleObject, UUID
from api.utils import add_scrobble, last_year_as_epoch, get_arts_scrobbles
from api.utils import get_arts_count, get_tracks_count, get_track_stats, get_art_stats
from scrobbles.models import Scrobble


class JsonpResponse(HttpResponse):

    def __init__(self, data, callback):
        json = jsondumps(data)
        jsonp = "%s(%s)" % (callback, json)
        HttpResponse.__init__(
            self, jsonp,
            mimetype='application/json'
            )


class UserAPI(APIObject):

    def do_GET(self):
        # Check privacy
        if self.user != self.authenticated_user:
            return self.unauthorized()
        ans = serializers.serialize('json', [self.user,], fields=('user_mane',
            'first_name', 'last_name', 'email', 'date_joined','last_login'))
        return HttpResponse(ans, mimetype="application/json")


class Scrobbles(APIObject):
    """
    URL: api/USER/scrobbles/
    Method: POST, GET
    """
    type_val = {
            'GET':{
                'from':int,
                'to':int,
                'limit':int,
                },
            'POST':{
                'timestamp':int,
                'art_mbid':UUID,
                'tit_mbid':UUID,
                'alb_mbid':UUID,
                }
            }
    mandatory = {
            'GET':None,
            'POST': ['timestamp', 'art', 'tit'],
            }

    def do_POST(self):
        if self.user != self.authenticated_user:
            return self.unauthorized()
        scrobble = ScrobbleObject(self.request.POST)
        if not scrobble.check_scrobble():
            return self.bad_request('Malformed scrobble')
        add_scrobble(scrobble, self.user)
        # TODO
        # Controls return value for process and return the serialized
        # object, with either a 200 ("OK") or a 201 ("Created") status code
        return self.ok()

    def do_GET(self):
        if self.user != self.authenticated_user:
            return self.unauthorized()
        dfrom, dto, limit, name = self.get_filters()
        scrobbles = Scrobble.objects.filter(user=self.user
                                   ).filter(date__range=(dfrom, dto)
                                   ).order_by('-date')
        if limit:
            scrobbles = scrobbles[:limit]
        detailed_sc = [sc.get_detailled_scrobble() for sc in scrobbles]
        if 'callback' in self.request.GET:
            return JsonpResponse(detailed_sc, self.request.GET['callback'])
        ans = jsondumps(detailed_sc)
        return HttpResponse(ans, mimetype="application/json")

    def validate(self):
        """
        Returns (Bool, message)
        If Boolean in False then message is not None and should contains info
        """
        method = self.request.method
        if method == 'GET':
            data = self.request.GET  # QueryDict Object
        elif method == 'POST':
            data = self.request.POST  # QueryDict Object
        ptypes = self.__class__.type_val.get(method)  # get parameters types
        mandat = self.__class__.mandatory.get(method)  # get non-optional values
        if mandat:
            if not (set(mandat) & set(data.dict().keys()) == set(mandat)):
                return False, u'Missing mandatory parameter'
        for param, value in data.iteritems():
            if param not in ptypes:
                continue
            ptype = ptypes.get(param)  # get expected param type
            try:
                ptype(value)
            except ValueError as err:
                return False, err
        return True, None


class ArtistsStat(APIObject):
    """
    URL: /api/USER/artists/
    Method: GET
    """
    type_val = {
            'GET':{
                'from':int,
                'to':int,
                'limit':int,
                },
            }

    def do_GET(self):
        if self.user != self.authenticated_user:
            return self.unauthorized()
        filters = self.get_filters(def_from=last_year_as_epoch())
        arts = get_arts_count(self.user, *filters)
        artists = [{'name':art.name,
                    'count':art.count,
                    'id':art.scid,
                    'is_mbid':art.is_mbid} for art in arts]
        if 'callback' in self.request.GET:
            return JsonpResponse(artists, self.request.GET['callback'])
        ans = jsondumps(artists)
        return HttpResponse(ans, mimetype="application/json")

    def validate(self):
        """
        Returns (Bool, message)
        If Boolean in False then message is not None and should contains info
        """
        data = self.request.GET  # QueryDict Object
        ptypes = self.__class__.type_val.get('GET')  # get parameters types
        for param, value in data.iteritems():
            if param not in ptypes:
                continue
            ptype = ptypes.get(param)  # get expected param type
            try:
                ptype(value)
            except ValueError as err:
                return False, err
        return True, None


class ArtistScrobbles(ArtistsStat):
    """
    URL: /api/USER/scrobbles/artists/<UUID>
    Method: GET
    """

    def do_GET(self):
        if self.user != self.authenticated_user:
            return self.unauthorized()
        dfrom, dto, limit, name = self.get_filters(def_from=last_year_as_epoch())
        art_sc = get_arts_scrobbles(self.user, dfrom, dto, self.uuid)
        art_sc = art_sc.order_by('-date')
        if limit:
            art_sc = art_sc[:limit]
        artist_sc = [{'track':sc.track.name,
                      'date':sc.date.strftime('%s'),} for sc in art_sc]
        if 'callback' in self.request.GET:
            return JsonpResponse(artist_sc, self.request.GET['callback'])
        ans = jsondumps(artist_sc)
        return HttpResponse(ans, mimetype="application/json")


class TracksStat(ArtistsStat):
    """
    URL: /api/USER/titles/
    Method: GET
    """

    def do_GET(self):
        if self.user != self.authenticated_user:
            return self.unauthorized()
        dfrom, dto, limit, name = self.get_filters(def_from=last_year_as_epoch())
        tracks = get_tracks_count(self.user, dfrom, dto)
        tracks = tracks.order_by('-count')
        if limit:
            tracks = tracks[:limit]
        tracks_list = [{'name':trk.name,
                        'count':trk.count,
                        'id':trk.scid,
                        'artist':trk.artist.name,} for trk in tracks]
        if 'callback' in self.request.GET:
            return JsonpResponse(tracks_list, self.request.GET['callback'])
        ans = jsondumps(tracks_list)
        return HttpResponse(ans, mimetype="application/json")


def apidoc(request):
    return render_to_response('api.html', context_instance=RequestContext(request))

@login_required
def accounts(request):
    wcounts = get_art_stats(request.user, week=True)
    dcounts = get_art_stats(request.user, day=True)
    most_played = get_track_stats(request.user)
    scrobbles = Scrobble.objects.filter(user=request.user).order_by('date')
    context = {'scrobbles': scrobbles.reverse()[:20],
            'week': {'count': wcounts,},
            'day': {'count': dcounts,},
            'mostplayed':most_played,
            }
    #raise Exception
    return render_to_response('accounts.html', dictionary=context, context_instance=RequestContext(request))

#user_api = UserAPI()
user_api = csrf_exempt(UserAPI())

#scrobbles = Scrobbles()
scrobbles = csrf_exempt(Scrobbles())

artist_scbbs = csrf_exempt(ArtistScrobbles())
artists_stat = csrf_exempt(ArtistsStat())
tracks_stat = csrf_exempt(TracksStat())

# vim: ai ts=4 sw=4 sts=4 expandtab
