#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
import threading

from time import sleep
from collections import deque

from client import scrobble, ScrobbleError

from lib.mpdclient import Player, PlayerError
from lib.daemon import Daemon

import logging

try:
    from config import USER, PWD, SCSERVER, MPDSERVER
except ImportError:
    logging.error("""Please create a config.py file with
            USER        = <scrobbler server user>
            PWD         = <scrobbler server pwd>
            SCSERVER    = <scrobbler server as FQDN or IP>
            MPDSERVER   = <MPD server as FQDN or IP>
            MPDPORT     = <MPD port, default to 6600 if not provided>
            LOGFILE     = <full path to log, default to /tmp/mpd_scrobbler.log>
            LOGLEVEL    = <level in (debug, info, warning, error), default to "info">
            """)
    sys.exit(1)

try:
    from config import MPDPORT
except ImportError:
    MPDPORT = 6600

try:
    from config import LOGFILE
except ImportError:
    LOGFILE = '/tmp/mpd_scrobbler.log'

try:
    from config import LOGLEVEL
except ImportError:
    LOGLEVEL = 'info'

LEVELS = {'debug': logging.DEBUG,
          'info': logging.INFO,
          'warning': logging.WARNING,
          'error': logging.ERROR,
          'critical': logging.CRITICAL}

logging.basicConfig(format='%(asctime)s  %(levelname)s  %(message)s', level=LEVELS.get(LOGLEVEL.lower()))

class QueuManagement(threading.Thread):

    def __init__(self, queu):
        threading.Thread.__init__(self)
        self.queu = queu
        self.queu_copy = deque(queu)

    def scrobble_queu(self):
        track = self.queu_copy.popleft()
        try:
            scrobble(trk=track, user=USER, pwd=PWD, srv=SCSERVER)
        except ScrobbleError as err:
            logging.warning('scrobbler: Failed catching up with "%s": %s' % (track, err))
            if hasattr(err, 'http_out'):
                logging.debug(u'scrobbler: http-error %s→\n%s' % (err.code, err.http_out))
            self.queu.appendleft(track)
            return False
        logging.info('Catching up with {0}'.format(track))
        self.queu.remove(track)
        return True

    def run(self):
        for i in xrange(len(self.queu_copy)):
            if not self.scrobble_queu():
                return


class MpdScrobbler(Daemon, Player):

    def __init__(self, pid, log):
        Player.__init__(self, MPDSERVER, MPDPORT)
        Daemon.__init__(self, pid, stdout=log, stderr=log)
        self.track_format = True
        self.current_track = None
        self.failed_sc_cache = deque()

    def run(self):
        try:
            self.connect()
            logging.info("Connected to %s" % MPDSERVER)
        except PlayerError as err:
            logging.error('player: {0}'.format(err))
            self.reconnect()
        except Exception as err:
            logging.error('player ukn err: {0}'.format(err))
            sys.exit(1)
        try:
            self.main()
        except KeyboardInterrupt:
            self.disconnect()
            sys.exit(0)

    def reconnect(self):
        """Trying to reconnect cycling through longer timeout
        cycle : 5s 10s 1m 5m 20m 1h
        """
        sleepfor = [5, 10, 60, 300, 1200, 3600]
        while True:
            tmp = sleepfor.pop(0)
            sleepfor.append(tmp)
            logging.info('trying to reconnect in {:>3d} seconds'.format(tmp))
            #logging.debug('trying to reconnect in {:03d} seconds'.format(tmp))
            sleep(tmp)
            try:
                self.connect()
            except PlayerError as err:
                continue
            except Exception as err:
                logging.error('Unhandled exception, quiting!')
                sys.exit(1)
            logging.info('Got reconnected')
            break

    def scrobble(self):
        if self.failed_sc_cache:
            logging.debug('Got %i in queu' % len(self.failed_sc_cache))
        logging.info(self.current_track)
        art = self.current_track.get_artist()
        tit = self.current_track.get_title()
        if not (art and tit):
            logging.warning(u'Artist or track name missing in %s, cannot scrobble!' %
                    self.current_track)
            return
        try:
            scrobble(trk=self.current_track, user=USER, pwd=PWD, srv=SCSERVER)
        except ScrobbleError as err:
            logging.error('scrobbler: {0}'.format(err))
            if hasattr(err, 'http_out'):
                logging.debug(u'scrobbler: http-error %s→\n%s' % (err.code, err.http_out))
            self.failed_sc_cache.append(self.current_track)
        else:
            # Deal with the queu if last scrobble went fine
            if self.failed_sc_cache:
                QueuManagement(self.failed_sc_cache).start()

    def log_track(self):
        current = self.currentsong()
        sleep(10)
        if self.currentsong() != current:
            # Not listenning to the same track long enought
            return
        if current != self.current_track:
            self.current_track = current
            self.scrobble()

    def main(self):
        if self.status().get('state') == 'play':
            self.log_track()  # scrobble at startup
        while True:
            try:
                self.idle()
            except PlayerError as err:
                logging.error('player: {0}'.format(err))
                self.reconnect()
            except Exception as err:
                logging.error('player ukn err: {0}'.format(err))
                sys.exit(1)
            if self.status().get('state') != 'play':
                logging.info('not playing')
                continue
            self.log_track()
            sys.stdout.flush()

    def shutdown(self):
        """Called on Dæmon stop"""
        logging.info(u"Stopping Daemon!")


def main():
    daemon = MpdScrobbler('/tmp/mpd_scrobbler.pid', LOGFILE)
    if len(sys.argv) == 2:
        if 'start' == sys.argv[1]:
            daemon.start()
        elif 'stop' == sys.argv[1]:
            daemon.stop()
        elif 'restart' == sys.argv[1]:
            daemon.restart()
        else:
            print "Unknown command"
            sys.exit(2)
        sys.exit(0)
    else:
        print "usage: %s start|stop|restart" % sys.argv[0]
        sys.exit(2)

# Script starts here
if __name__ == '__main__':
    main()

# vim: ai ts=4 sw=4 sts=4 expandtab
